#include <iostream>
using namespace std;

#include <MainWindow.hh>

MainWindow::MainWindow(MsgThread *t)
{
	msgThread = t;
	automaticImgSave = false;

	statusBar = new QStatusBar;

	menuFile = menuBar()->addMenu("&File");
	menuHelp = menuBar()->addMenu("&Help");

	actionSaveStreamPlan = new QAction("&Save Streams Plan...", this);
	menuFile->addAction(actionSaveStreamPlan);
	actionSaveStreamPlan->setEnabled(false);
	actionSaveTreePlan = new QAction("&Save Tree Plan...", this);
	menuFile->addAction(actionSaveTreePlan);
	actionSaveTreePlan->setEnabled(false);
	actionExit = new QAction("&Exit", this);
	menuFile->addAction(actionExit);
	actionContents = new QAction("&Contents", this);
	menuHelp->addAction(actionContents);
	actionAbout = new QAction("&About", this);
	menuHelp->addAction(actionAbout);

	mainArea = new QWidget;
	leftArea = new QWidget;
	rightArea = new QWidget;
	ontoArea = new QGroupBox("Ontology requests");
	systemArea = new QGroupBox("System messages");
	planArea = new QGroupBox("Plan result");

	mainLayout = new QHBoxLayout;
	leftLayout = new QVBoxLayout;	
	rightLayout = new QVBoxLayout;

	ontoLayout = new QVBoxLayout;
	systemLayout = new QVBoxLayout;
	planLayout = new QVBoxLayout;
	
	planTab = new QTabWidget;
	
	systemText = new QTextEdit;
	ontoText = new QTextEdit;
	planText = new QTextEdit;
	streamView = new SvgView;
	treeView = new SvgView;
	
	buttonNetwork = new QPushButton(" network");
	iconNetOn = new QIcon(QCoreApplication::applicationDirPath()+"/led-green.gif");
	iconNetOff = new QIcon(QCoreApplication::applicationDirPath()+"/led-red.gif");

	mainArea->setLayout(mainLayout);
	leftArea->setLayout(leftLayout);
	rightArea->setLayout(rightLayout);

	ontoArea->setLayout(ontoLayout);
	systemArea->setLayout(systemLayout);
	planArea->setLayout(planLayout);

	mainLayout->addWidget(leftArea);
	mainLayout->addWidget(rightArea);
	leftLayout->addWidget(ontoArea);
	rightLayout->addWidget(systemArea);
	rightLayout->addWidget(planArea);

	ontoLayout->addWidget(ontoText);
	systemLayout->addWidget(systemText);
	
	planTab->addTab(planText, "Plan");
	planTab->addTab(streamView, "Streams");
	planTab->addTab(treeView, "Tree");
	planLayout->addWidget(planTab);
	
	this->setCentralWidget(mainArea);
	this->setStatusBar(statusBar);

	mainArea->setMinimumHeight(600);
	leftArea->setMinimumWidth(420);
	leftArea->setMaximumWidth(420);
	rightArea->setMinimumWidth(650);

	systemText->setReadOnly(true);
	systemText->setFontPointSize(9);
	ontoText->setReadOnly(true);
	ontoText->setFontPointSize(8);
	
	// buttonNetwork
	buttonNetwork->setIcon(*iconNetOn);
	buttonNetwork->setFlat(true);
	statusBar->addPermanentWidget(buttonNetwork);

	connect(actionExit, SIGNAL(triggered()), this, SLOT(close()));
	connect(actionSaveStreamPlan, SIGNAL(triggered()), this, SLOT(saveStreamPlanImage()));
	connect(actionSaveTreePlan, SIGNAL(triggered()), this, SLOT(saveTreePlanImage()));
	connect(actionAbout, SIGNAL(triggered()), this, SLOT(about()));
	connect(buttonNetwork, SIGNAL(clicked()), this, SLOT(networkInterrupt()));
	connect(msgThread, SIGNAL(sendStop()), this, SLOT(close()));
	connect(msgThread, SIGNAL(sendIni(QString)), this, SLOT(updateStatusBar(QString)));
	connect(msgThread, SIGNAL(sendMsg(QString)), this, SLOT(updateSystemMessage(QString)));
	connect(msgThread, SIGNAL(planImg(QString, QString)), this, SLOT(changePlanImages(QString, QString)));
	connect(msgThread, SIGNAL(planTxt(QString)), this, SLOT(changePlanText(QString)));
	connect(msgThread, SIGNAL(planRes(QString, QColor)), this, SLOT(updatePlanResult(QString, QColor)));
	connect(msgThread, SIGNAL(sendReqP(QString, QString, QString, QString)), this, SLOT(addPlanRequest(QString, QString, QString, QString)));
	connect(msgThread, SIGNAL(sendReq1(QString, QString, QString)), this, SLOT(updateSingleOntoRequest(QString, QString, QString)));
	connect(msgThread, SIGNAL(sendReq2(QString, QString, QString, QString)), this, SLOT(updateAgentOntoRequest(QString, QString, QString, QString)));
	connect(msgThread, SIGNAL(planID(QString)), this, SLOT(updateCurrentPlanID(QString)));
}

// ----- FUNCTIONS -----

void MainWindow::addSystemMessage(string const& s)
{
	this->insertPlainText(systemText, QString::fromStdString(s) + "\n");
}

void MainWindow::setApplicationTitle(string const& s)
{
	this->setWindowTitle(QString::fromStdString(s));
}

void MainWindow::setStatusMessage(string const& s)
{
	statusBar->showMessage(QString::fromStdString(s));
}

void MainWindow::setAutomaticImgSave(bool f) {
	this->automaticImgSave = f;
}

// ----- SLOTS -----

void MainWindow::updateCurrentPlanID(const QString &qs) {
	this->currentPlanID = qs;
}

void MainWindow::saveStreamPlanImage() {
	QString imgFile = QFileDialog::getSaveFileName(this, "Save", QString(), "Images (*.png *.gif *.jpg *.jpeg)");
	if(currentStreamImgFile != NULL) {
		QSvgRenderer svgRenderer(currentStreamImgFile);
		QPixmap pixmap(svgRenderer.defaultSize().width(), svgRenderer.defaultSize().height());
		QPainter painter(&pixmap);
		svgRenderer.render(&painter);
		painter.end();
		pixmap.save(imgFile, 0, -1);
	}
}

void MainWindow::saveTreePlanImage() {
	QString imgFile = QFileDialog::getSaveFileName(this, "Save", QString(), "Images (*.png *.gif *.jpg *.jpeg)");
	if(currentTreeImgFile != NULL) {
		QSvgRenderer svgRenderer(currentTreeImgFile);
		QPixmap pixmap(svgRenderer.defaultSize().width(), svgRenderer.defaultSize().height());
		QPainter painter(&pixmap);
		svgRenderer.render(&painter);
		painter.end();
		pixmap.save(imgFile, 0, -1);
	}
}

void MainWindow::networkInterrupt() {
	if(buttonNetwork->icon().cacheKey() == iconNetOn->cacheKey()) {
		buttonNetwork->setIcon(*iconNetOff);
		msgThread->changeActivityNetwork(false);
	}
	else {
		buttonNetwork->setIcon(*iconNetOn);
		msgThread->changeActivityNetwork(true);
	}
}

void MainWindow::about() {
	QMessageBox::about(this, "About hatpconsole", "<H1> hatpconsole </H1>"
		"The <b>hatpconsole</b> application is a viewer dedicated to HATPOnboard "
		"allowing to verify messages exchanged between the planner and the supervisor.<br /><br />"
		"created by Julien Guitton < br />"
		"copyright 2010/2011 LAAS/CNRS <br />"
		"<i>julien.guitton@laas.fr</i>"
	);
}

void MainWindow::updateSystemMessage(const QString &qs) 
{
	this->insertPlainText(systemText, qs + "\n");
	systemText->verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);

}

void MainWindow::updateStatusBar(const QString &qs) {
	statusBar->showMessage(qs);
}

void MainWindow::changePlanImages(const QString &qs1, const QString &qs2) {
	QString streamFileName(qs1);
	if (!streamFileName.isEmpty()) {
		QFile file(streamFileName);
		if (!file.exists()) {
			std::cerr<<"ERROR: File "<<streamFileName.toStdString()<<" does not exist"<<std::endl;
			return;
		}

		streamView->openFile(file);
	}else{
		std::cerr<<"ERROR: Empty stream filename"<<std::endl;
		return;
	}

	QString treeFileName(qs2);
	if (!treeFileName.isEmpty()) {
		QFile file(treeFileName);
		if (!file.exists()) {
			std::cerr<<"ERROR: File "<<treeFileName.toStdString()<<" does not exist"<<std::endl;
			return;
		}

		treeView->openFile(file);
	}else{
		std::cerr<<"ERROR: Empty tree filename"<<std::endl;
		return;
	}
}

void MainWindow::insertPlainText(QTextEdit* area, QString const& text, QColor color) {
	QTextCursor* cursor = new QTextCursor(area->textCursor());
	cursor->movePosition(QTextCursor::End, QTextCursor::MoveAnchor, 1);
	QTextCharFormat* format = new QTextCharFormat(area->currentCharFormat());
	format->setForeground(QBrush(color));
	cursor->insertText(text, *format);
}

void MainWindow::addPlanRequest(const QString &qy, const QString &qi, const QString &qa, const QString &qp) {
	actionSaveStreamPlan->setEnabled(false);
	actionSaveTreePlan->setEnabled(false);
	streamView->clear();
	treeView->clear();
	planText->clear();
	ontoText->clear();
	this->insertPlainText(systemText, "\n");
	this->insertPlainText(systemText, "New planning request received: {\n", Qt::black);
	this->insertPlainText(systemText, "  Id = ", Qt::black);
	this->insertPlainText(systemText, qi + "\n", Qt::darkBlue);
	this->insertPlainText(systemText, "  Type = ", Qt::black);
	this->insertPlainText(systemText, qy + "\n", Qt::darkBlue);
	this->insertPlainText(systemText, "  Task = ", Qt::black);
	this->insertPlainText(systemText, qa + "\n", Qt::darkBlue);
	this->insertPlainText(systemText, "  Params = ", Qt::black);
	this->insertPlainText(systemText, "[" + qp + "]\n", Qt::darkBlue);
	this->insertPlainText(systemText, "}\n", Qt::black);
	systemText->verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);

}

void MainWindow::updatePlanResult(const QString &qm, const QColor &qc) {
	this->insertPlainText(systemText, "Planning result: ", Qt::black);
	this->insertPlainText(systemText, qm + "\n", qc);
	systemText->verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);
}

void MainWindow::updateSingleOntoRequest(const QString &qo, const QString &qp, const QString &qv) {
	this->insertPlainText(ontoText, qo + " ", Qt::black);
	this->insertPlainText(ontoText, qp + " ", Qt::darkCyan);
	if(qv == "") this->insertPlainText(ontoText, qv + "null\n", Qt::darkRed);
	else this->insertPlainText(ontoText, qv + "\n", Qt::darkGreen);
	ontoText->verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);
}

void MainWindow::updateAgentOntoRequest(const QString &qa, const QString &qo, const QString &qp, const QString &qv) {
	this->insertPlainText(ontoText, "(" + qa + "): ", Qt::darkGray);
	this->updateSingleOntoRequest(qo, qp, qv);
}

void MainWindow::changePlanText(const QString &qm) {
	this->insertPlainText(planText, qm + "\n", Qt::black);
	planText->verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);
}


// ----- EVENTS -----

void MainWindow::closeEvent(QCloseEvent *event) {
	msgThread->stopListening();
	event->accept();
}
