#ifndef HEADER_MAINWINDOW
#define HEADER_MAINWINDOW

#include <QtGui>
#include <QtSvg>
#include <string>

#include <MsgThread.hh>
#include <SvgView.hh>

class MainWindow : public QMainWindow
{
	Q_OBJECT

	private:

		QStatusBar *statusBar;
		QMenu *menuFile;
		QMenu *menuHelp;
		QAction *actionSaveStreamPlan;
		QAction *actionSaveTreePlan;
		QAction *actionExit;
		QAction *actionContents;
		QAction *actionAbout;

		QHBoxLayout *mainLayout;
		QVBoxLayout *leftLayout;
		QVBoxLayout *rightLayout;
		QVBoxLayout *ontoLayout;
		QVBoxLayout *systemLayout;
		QVBoxLayout *planLayout;

		QWidget *mainArea;
		QWidget *leftArea;
		QWidget *rightArea;

		QGroupBox *ontoArea;
		QGroupBox *systemArea;
		QGroupBox *planArea;

		QTabWidget *planTab;

		QTextEdit *systemText;
		QTextEdit *ontoText;
		QTextEdit *planText;
		SvgView *streamView;
		SvgView *treeView;

		MsgThread *msgThread;
		QString currentStreamImgFile;
		QString currentTreeImgFile;

		QPushButton *buttonNetwork;
		QIcon *iconNetOn;
		QIcon *iconNetOff;
		
		bool automaticImgSave;
		QString currentPlanID;

	protected:

		void closeEvent(QCloseEvent *event);

	public:

		MainWindow(MsgThread *t);
		void addSystemMessage(std::string const& s);
		void setApplicationTitle(std::string const& s);
		void setStatusMessage(std::string const& s);
		void insertPlainText(QTextEdit* area, QString const& text, QColor color = Qt::black);
		void setAutomaticImgSave(bool f);

	private slots:

		void saveStreamPlanImage();
		void saveTreePlanImage();
		void networkInterrupt();
		void about();
		void updateSystemMessage(const QString &qs);
		void updateStatusBar(const QString &qs);
		void changePlanImages(const QString &qs1, const QString &qs2);
		void changePlanText(const QString &qs);
		void updatePlanResult(const QString &qm, const QColor &qc);
		void addPlanRequest(const QString &qa, const QString &qo, const QString &qp, const QString &qv);
		void updateSingleOntoRequest(const QString &qo, const QString &qp, const QString &qv);
		void updateAgentOntoRequest(const QString &qa, const QString &qo, const QString &qp, const QString &qv);
		void updateCurrentPlanID(const QString &qs);

};

#endif

