#include <MinimalWindow.hh>
#include <SvgView.hh>

MinimalWindow::MinimalWindow(MsgThread *t):
	QMainWindow(), msgThread(t), view(new SvgView), statusBarContent(new QLabel("Waiting for new Plan", this))
{
	QMenu *fileMenu = new QMenu(tr("&File"), this);
	QAction *quitAction = fileMenu->addAction(tr("E&xit"));
	quitAction->setShortcut(QKeySequence(tr("Ctrl+Q")));
	quitAction->setShortcuts(QKeySequence::Quit);
	connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
	menuBar()->addMenu(fileMenu);

	setCentralWidget(view);
	
	statusBar()->addWidget(statusBarContent);
	
	connect(msgThread, SIGNAL(planImg(QString, QString)), this, SLOT(changePlanImages(QString, QString)));
	connect(msgThread, SIGNAL(planRes(QString, QColor)), this, SLOT(updatePlanResult(QString, QColor)));
	connect(msgThread, SIGNAL(sendReqP(QString, QString, QString, QString)), this, SLOT(addPlanRequest(QString, QString, QString, QString)));

	resize(1000, 500);
	setMinimumSize(1000, 500);
}

void MinimalWindow::setApplicationTitle(std::string const& s)
{
	this->setWindowTitle(QString::fromStdString(s));
}

void MinimalWindow::changePlanImages(const QString &qs1, const QString &) {//Ignore second parameter containing the plan tree
	QString fileName(qs1);
	if (!fileName.isEmpty()) {
		QFile file(fileName);
		if (!file.exists()) {
			std::cerr<<"ERROR: File "<<fileName.toStdString()<<" does not exist"<<std::endl;
			return;
		}

		view->openFile(file);
	}else{
		std::cerr<<"ERROR: Empty filename"<<std::endl;
		return;
	}
}

void MinimalWindow::updatePlanResult(const QString &qm, const QColor &qc) {
	statusBarContent->setText(qm);
}

void MinimalWindow::addPlanRequest(const QString &qy, const QString &qi, const QString &qa, const QString &qp) {
	view->clear();
	statusBarContent->setText("Waiting for new Plan");
}

void MinimalWindow::closeEvent(QCloseEvent *event) {
	msgThread->stopListening();
	event->accept();
}
